﻿using Infragistics.Controls.Reports;
using Infragistics.Reports;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;

namespace IGReportHideRow
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        public static int activeExporters;
        private static readonly object _locker = new object();
        public static ObservableCollection<Address> Addresses { get; set; }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            Export();
        }

        private void Export()
        {
            Addresses = new ObservableCollection<Address>();

            Address temp = new Address();
            temp.Address1 = "123 Main St";
            temp.Address2 = null;
            temp.City = "Detroit";
            temp.State = "MI";
            temp.Zip = "01234";
            Addresses.Add(temp);

            FileStream stream = null;
            string folder = @"C:\temp";
            string name = "addressPDF.pdf";
            if (!Directory.Exists(folder))
            {
                Directory.CreateDirectory(folder);
            }
            string filepath = Path.Combine(folder, name);

            Uri reportUri = new Uri("/IGReportHideRow;component/Report1.igr", UriKind.Relative);
            stream = File.OpenWrite(filepath);

            IClientExporter clientExporter = xamReportViewer1.CreateExporter(reportUri);
            clientExporter.ExportCompleted += exporter_ExportCompleted;
            clientExporter.ExportCanceled += exporter_ExportCanceled;

            lock (_locker)
            {
                activeExporters += 1;
            }

            clientExporter.ExportAsync(stream, "PDF");
        }

        private void Process()
        {
            lock (_locker)
            {
                Export();

                while (activeExporters > 0)
                {
                    Monitor.Wait(_locker);
                }
            }
        }

        /// <summary>
        /// Performed if report is cancelled
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private static void exporter_ExportCanceled(object sender, Infragistics.Reports.ExportCanceledEventArgs e)
        {
            // Do Something with the error
            ReleaseResources((IClientExporter)sender, (FileStream)e.Stream, true);
        }

        /// <summary>
        /// Performed upon completion of report export
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private static void exporter_ExportCompleted(object sender, Infragistics.Reports.ExportCompletedEventArgs e)
        {
            ReleaseResources((IClientExporter)sender, (FileStream)e.Stream, false);
        }

        /// <summary>
        /// Release resources
        /// </summary>
        /// <param name="clientExporter">exporter</param>
        /// <param name="stream">File Stream</param>
        /// <param name="deleteFile">True if need to delete the file</param>
        private static void ReleaseResources(IClientExporter clientExporter, FileStream stream, bool deleteFile)
        {
            if (clientExporter != null)
            {
                clientExporter.ExportCompleted -= exporter_ExportCompleted;
                clientExporter.ExportCanceled -= exporter_ExportCanceled;
                clientExporter.Dispose();
            }

            if (stream != null)
            {
                stream.Close();

                if (deleteFile)
                {
                    File.Delete(stream.Name);
                }
            }

            lock (_locker)
            {
                activeExporters -= 1;
                Monitor.Pulse(_locker);
            }
        }

    }

    public class Address
    {
        public string Address1 { get; set; }
        public string Address2 { get; set; }
        public string City { get; set; }
        public string State { get; set; }
        public string Zip { get; set; }
    }
}
